@@ -136,11 +136,46 @@ public:
 		m_maxParents = maxpars;
 		m_maxCategories = maxcats;
 
-		m_numParents = (int*) CATNET_MALLOC(m_numNodes * sizeof(int));
-		memset(m_numParents, 0, m_numNodes * sizeof(int));
-		m_parents = (int**) CATNET_MALLOC(m_numNodes * sizeof(int*));
+		m_numParents    = (int*)   CATNET_MALLOC(m_numNodes * sizeof(int));
+		
+		m_parents       = (int**)  CATNET_MALLOC(m_numNodes * sizeof(int*));
+		
+		m_numCategories = (int*)   CATNET_MALLOC(m_numNodes * sizeof(int));
+
+		m_catIndices    = (int**)  CATNET_MALLOC(m_numNodes * sizeof(int*));		
+
+ 		m_nodeNames     = (char**) CATNET_MALLOC(m_numNodes * sizeof(char*));
+
+		m_pProbLists    = (PROB_LIST<t_prob>**) CATNET_MALLOC(m_numNodes * sizeof(PROB_LIST<t_prob>*));
+
+		if (!m_numParents || !m_parents || !m_numCategories || !m_catIndices 
+                 || !m_nodeNames || !m_pProbLists) {
+			if (m_numParents)
+				CATNET_FREE(m_numParents);
+			if (m_parents)
+				CATNET_FREE(m_parents);
+			if (m_numCategories)
+				CATNET_FREE(m_numCategories);
+			if (m_nodeNames)
+				CATNET_FREE(m_nodeNames);
+			if(m_catIndices)
+				CATNET_FREE(m_catIndices);
+			if (m_pProbLists)
+				CATNET_FREE(m_pProbLists);
+			m_numNodes      = 0;
+			m_maxParents    = 0;
+			m_maxCategories = 0;
+			m_nodeNames     = 0;
+			m_numParents    = 0;
+			m_parents       = 0;
+			m_numCategories = 0;
+			m_catIndices    = 0;
+			m_pProbLists    = 0;
+			return;
+		}
+
 		memset(m_parents, 0, m_numNodes * sizeof(int*));
-		m_numCategories = (int*) CATNET_MALLOC(m_numNodes * sizeof(int));
+
 		if (pcats != 0)
 			memcpy(m_numCategories, pcats, m_numNodes * sizeof(int));
 		else {
@@ -148,10 +183,8 @@ public:
 				m_numCategories[i] = m_maxCategories;
 		}
 
-		m_catIndices = (int**) CATNET_MALLOC(m_numNodes * sizeof(int*));
 		memset(m_catIndices, 0, m_numNodes * sizeof(int*));
-		
-		m_nodeNames = (char**) CATNET_MALLOC(m_numNodes * sizeof(char*));
+
 		if (nodes) {	
 			for (i = 0; i < m_numNodes; i++) {
 				m_nodeNames[i] = 0;
@@ -159,40 +192,47 @@ public:
 					continue;
 				nodenamelen = strlen(nodes[i]);
 				m_nodeNames[i] = (char*) CATNET_MALLOC((nodenamelen+1) * sizeof(char));
-				strcpy(m_nodeNames[i], nodes[i]);
+				if (m_nodeNames[i])
+					strcpy(m_nodeNames[i], nodes[i]);
 			}
 		}
 		else {
 			memset(m_nodeNames, 0, m_numNodes * sizeof(char*));
 		}
 
+		memset(m_numParents, 0, m_numNodes * sizeof(int));
 		if (pnumpars != 0 && ppars != 0) {
 			memcpy(m_numParents, pnumpars, m_numNodes * sizeof(int));
 			for (i = 0; i < m_numNodes; i++) {
+				if (m_numParents[i] <= 0) 
+					continue;
 				m_parents[i] = (int*) CATNET_MALLOC(m_numParents[i] * sizeof(int));
-				memset(m_parents[i], 0, m_numParents[i] * sizeof(int));
-				if(ppars[i])
-					memcpy(m_parents[i], ppars[i], m_numParents[i] * sizeof(int));
+				if (m_parents[i]) {
+					memset(m_parents[i], 0, m_numParents[i] * sizeof(int));
+					if(ppars[i])
+						memcpy(m_parents[i], ppars[i], m_numParents[i] * sizeof(int));
+				}
 				if (m_numParents[i] > m_maxParents)
 					m_maxParents = m_numParents[i];
 			}
 		}
 		
-		nodeparcats = (int*)CATNET_MALLOC((m_maxParents+1)*sizeof(int));
-		m_pProbLists = (PROB_LIST<t_prob>**) CATNET_MALLOC(m_numNodes * sizeof(PROB_LIST<t_prob>*));
 		memset(m_pProbLists, 0, m_numNodes * sizeof(PROB_LIST<t_prob>*));
-		for (i = 0; i < m_numNodes; i++) {
-			if (pprobs && pprobs[i]) {
-				m_pProbLists[i] = new PROB_LIST<t_prob>;
-				*m_pProbLists[i] = *pprobs[i];
-			}
-			else {
-				for(j = 0; j < m_numParents[i]; j++)
-					nodeparcats[j] = m_numCategories[m_parents[i][j]]; 
-				m_pProbLists[i] = new PROB_LIST<t_prob>(m_numCategories[i], m_maxCategories, m_numParents[i], nodeparcats);
+		nodeparcats = (int*)CATNET_MALLOC((m_maxParents+1)*sizeof(int));
+		if (nodeparcats) {	
+			for (i = 0; i < m_numNodes; i++) {
+				if (pprobs && pprobs[i]) {
+					m_pProbLists[i] = new PROB_LIST<t_prob>;
+					*m_pProbLists[i] = *pprobs[i];
+				}
+				else {
+					for(j = 0; j < m_numParents[i]; j++)
+						nodeparcats[j] = m_numCategories[m_parents[i][j]]; 
+					m_pProbLists[i] = new PROB_LIST<t_prob>(m_numCategories[i], m_maxCategories, m_numParents[i], nodeparcats);
+				}
 			}
+			CATNET_FREE(nodeparcats);
 		}
-		CATNET_FREE(nodeparcats);
 	}
 
 	void setNodeNames(char **pnames, const int *porder) {
@@ -202,7 +242,8 @@ public:
 			return;
 		if(!m_nodeNames) {
 			m_nodeNames = (char**) CATNET_MALLOC(m_numNodes * sizeof(char*));
-			memset(m_nodeNames, 0, m_numNodes * sizeof(char*));
+			if (m_nodeNames)
+				memset(m_nodeNames, 0, m_numNodes * sizeof(char*));
 		}
 		for (i = 0; i < m_numNodes; i++) {
 			m_nodeNames[i] = 0;
@@ -212,7 +253,8 @@ public:
 			if(m_nodeNames[i])
 				CATNET_FREE(m_nodeNames[i]);
 			m_nodeNames[i] = (char*) CATNET_MALLOC((strlen(str)+1) * sizeof(char));
-			strcpy((char*)m_nodeNames[i], str);
+			if (m_nodeNames[i] && str)
+				strcpy((char*)m_nodeNames[i], str);
 		}
 	}
 
@@ -233,7 +275,8 @@ public:
 			if(m_nodeNames[i])
 				CATNET_FREE(m_nodeNames[i]);
 			m_nodeNames[i] = (char*) CATNET_MALLOC((strlen(str)+1) * sizeof(char));
-			strcpy((char*)m_nodeNames[i], str);
+			if (m_nodeNames[i] && str)
+				strcpy((char*)m_nodeNames[i], str);
 		}
 	}
 
@@ -242,10 +285,12 @@ public:
 		// reset node names only
 		if (!pNumCats || !pCatIndices || !m_numCategories) 
 			return;
-		//if(m_catIndices)
-		//	CATNET_FREE(m_catIndices);
-		if(!m_catIndices)
+		if(!m_catIndices) {
 			m_catIndices = (int**) CATNET_MALLOC(m_numNodes * sizeof(int*));
+			memset(m_catIndices, 0, m_numNodes * sizeof(int*));
+		}
+		if(!m_catIndices)
+			return;
 		for (i = 0; i < m_numNodes; i++) {
 			if(pNumCats[i] != m_numCategories[i])
 				break;
@@ -262,13 +307,19 @@ public:
 			delete m_pProbLists[node];
 		if (m_numParents[node] > m_maxParents)
 			m_maxParents = m_numParents[node];
-		if (!m_pProbLists)
+		if (!m_pProbLists) {
 			m_pProbLists = (PROB_LIST<t_prob>**) CATNET_MALLOC(m_numNodes
 					* sizeof(PROB_LIST<t_prob>*));
+			memset(m_pProbLists, 0, m_numNodes * sizeof(PROB_LIST<t_prob>*));
+		}
+		if(!m_pProbLists)
+			return;
 		m_pProbLists[node] = 0;
 		if (m_numParents[node] < 0 || m_numParents[node] > m_maxParents)
 			return;
 		int *parcats = (int*) CATNET_MALLOC(m_maxParents * sizeof(int));
+		if (!parcats)
+			return;
 		for (int i = 0; i < m_numParents[node]; i++) {
 			parcats[i] = m_numCategories[m_parents[node][i]];
 		}
@@ -335,16 +386,23 @@ public:
 				CATNET_FREE(m_parents[node]);
 			m_parents[node] = (int*) CATNET_MALLOC(m_numParents[node] * sizeof(int));
 		}
+		if (!m_parents[node])
+			return 0;
 		memcpy(m_parents[node], parents, m_numParents[node] * sizeof(int));
 
 		if (!m_pProbLists)
 			m_pProbLists = (PROB_LIST<t_prob>**) CATNET_MALLOC(m_numNodes * sizeof(PROB_LIST<t_prob>*));
-		else if(m_pProbLists[node])
-			delete m_pProbLists[node];
+		else {
+			if(m_pProbLists[node])
+				delete m_pProbLists[node];
+			m_pProbLists[node] = 0;
+		}
 
 		int *parcats = (int*)CATNET_MALLOC(m_numParents[node]*sizeof(int));
+		if (!parcats)
+			return 0;
 		for(int i = 0; i < m_numParents[node]; i++) 
-			parcats[i] = m_numCategories[parents[i]];	
+			parcats[i] = m_numCategories[parents[i]];
 		m_pProbLists[node] = new PROB_LIST<t_prob>(m_numCategories[node], m_maxCategories, m_numParents[node], parcats);
 		CATNET_FREE(parcats);
 		             
@@ -484,7 +542,10 @@ public:
 			}
 
 			paux = (int*) CATNET_MALLOC((poolsize + parpoolsize + 1) * sizeof(int));
-			if (poolsize > 0 && ppool)
+			if (!paux) 
+				break;
+
+			if (ppool && poolsize > 0)
 				memcpy(paux, ppool, poolsize * sizeof(int));
 			if (parpoolsize > 0 && parpool)
 				memcpy(paux + poolsize, parpool, parpoolsize * sizeof(int));
@@ -526,6 +587,8 @@ public:
 		parpool = findParentPool(nnode, parpoolsize);
 		// add nnode to the parents list
 		paux = (int*) CATNET_MALLOC((parpoolsize + 1) * sizeof(int));
+		if (!paux)
+			return 0;
 		if (parpool && parpoolsize > 0) 
 			memcpy(paux, parpool, parpoolsize * sizeof(int));
 		if (parpool)
@@ -534,10 +597,15 @@ public:
 		parpoolsize++;
 		parpool = paux;
 
-		pcats = (int*) CATNET_MALLOC(m_maxParents * sizeof(int));
-		paridx = (int*) CATNET_MALLOC(parpoolsize * sizeof(int));
+		pcats     = (int*) CATNET_MALLOC(m_maxParents * sizeof(int));
+		paridx    = (int*) CATNET_MALLOC(parpoolsize  * sizeof(int));
+		blocksize = (int*) CATNET_MALLOC(parpoolsize  * sizeof(int));
+
+		if (!pcats || !paridx || !blocksize) {
+			CATNET_FREE(parpool);
+			return 0;
+		}
 
-		blocksize = (int*) CATNET_MALLOC(parpoolsize * sizeof(int));
 		blocksize[parpoolsize - 1] = 1;
 		for (i = parpoolsize - 2; i >= 0; i--) {
 			blocksize[i] = blocksize[i + 1] * m_numCategories[parpool[i + 1]];
@@ -549,8 +617,14 @@ public:
 		}
 
 		jointprob = (t_prob*) CATNET_MALLOC(jointprobsize * sizeof(t_prob));
-		if (!jointprob)
-			return jointprob;
+		if (!jointprob) {
+			CATNET_FREE(pcats);
+			CATNET_FREE(paridx);
+			CATNET_FREE(blocksize);
+			CATNET_FREE(parpool);
+			return 0;
+		}
+
 		for (i = 0; i < jointprobsize; i++)
 			jointprob[i] = 1.0;
 
@@ -612,8 +686,8 @@ public:
 
 		CATNET_FREE(pcats);
 		CATNET_FREE(paridx);
-		CATNET_FREE(parpool);
 		CATNET_FREE(blocksize);
+		CATNET_FREE(parpool);
 
 		return jointprob;
 	}
@@ -623,7 +697,9 @@ public:
 		t_prob *jointprob, *margprob;
 		int jointprobsize = 0, nodecats, ic, k;
 		nodecats = m_numCategories[nnode];
-		margprob = (t_prob*) CATNET_MALLOC(nodecats * sizeof(t_prob));
+		margprob  = (t_prob*) CATNET_MALLOC(nodecats * sizeof(t_prob));
+		if (!margprob)
+			return 0;
 		jointprob = findJointProb(nnode, jointprobsize);
 		if (!jointprob)
 			return 0;
@@ -647,6 +723,8 @@ public:
 
 		porder = (int*)CATNET_MALLOC(m_numNodes * sizeof(int));
 		pfound = (int*)CATNET_MALLOC(m_numNodes * sizeof(int));
+		if (!porder || !pfound)
+			return 0;
 		memset(pfound, 0, m_numNodes * sizeof(int));
 		for(i = 0; i < m_numNodes; i++) {
 			for(j = 0; j < m_numNodes; j++) {
